@@ -17,9 +17,12 @@
 package org.jbpm.workbench.pr.client.editors.instance.diagram;
 
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.List;
+import java.util.Map;
 
 import com.google.gwtmockito.GwtMockitoTestRunner;
+import org.jbpm.workbench.common.preferences.ManagePreferences;
 import org.jbpm.workbench.pr.client.resources.i18n.Constants;
 import org.jbpm.workbench.pr.events.ProcessInstanceSelectionEvent;
 import org.jbpm.workbench.pr.model.NodeInstanceSummary;
@@ -34,19 +37,24 @@ import org.junit.Before;
 import org.junit.Test;
 import org.junit.runner.RunWith;
 import org.kie.api.runtime.process.ProcessInstance;
+import org.kie.workbench.common.widgets.client.popups.alert.AlertPopupView;
 import org.mockito.ArgumentCaptor;
 import org.mockito.InjectMocks;
 import org.mockito.Mock;
+import org.uberfire.client.views.pfly.widgets.ConfirmPopup;
 import org.uberfire.mocks.CallerMock;
 import org.uberfire.mocks.EventSourceMock;
 import org.uberfire.workbench.events.NotificationEvent;
+import org.uberfire.mvp.Command;
 
+import static com.google.common.collect.Maps.immutableEntry;
 import static java.util.Collections.emptyList;
 import static java.util.Collections.singletonList;
 import static org.assertj.core.api.Assertions.assertThat;
 import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertTrue;
 import static org.mockito.Matchers.any;
+import static org.mockito.Matchers.eq;
 import static org.mockito.Mockito.anyString;
 import static org.mockito.Mockito.never;
 import static org.mockito.Mockito.times;
@@ -68,6 +76,15 @@ public class ProcessInstanceDiagramPresenterTest {
     @Mock
     ProcessInstanceDiagramView view;
 
+    @Mock
+    protected ManagePreferences preferences;
+
+    @Mock
+    ConfirmPopup confirmPopup;
+
+    @Mock
+    AlertPopupView alertPopup;
+
     @InjectMocks
     ProcessInstanceDiagramPresenter presenter;
 
@@ -78,7 +95,7 @@ public class ProcessInstanceDiagramPresenterTest {
 
     @Test
     public void testEmptyProcessInstanceDiagram() {
-        when(processService.getProcessInstanceDiagramSummary(any())).thenReturn(
+        when(processService.getProcessInstanceDiagramSummary(any(), any(), any(), any())).thenReturn(
                 ProcessInstanceDiagramSummary.builder().withProcessDefinition(ProcessSummary.builder().nodes(emptyList()).timers(emptyList()).build()).withNodeInstances(emptyList()).withTimerInstances(emptyList()).build(),
                 ProcessInstanceDiagramSummary.builder().withProcessDefinition(ProcessSummary.builder().nodes(emptyList()).timers(emptyList()).build()).withNodeInstances(emptyList()).withTimerInstances(emptyList()).withSvgContent("").build()
         );
@@ -92,7 +109,7 @@ public class ProcessInstanceDiagramPresenterTest {
     @Test
     public void testProcessInstanceDiagram() {
         final String svgContent = "<svg></svg>";
-        when(processService.getProcessInstanceDiagramSummary(any())).thenReturn(
+        when(processService.getProcessInstanceDiagramSummary(any(), any(), any(), any())).thenReturn(
                 ProcessInstanceDiagramSummary.builder().withProcessDefinition(ProcessSummary.builder().nodes(emptyList()).timers(emptyList()).build()).withNodeInstances(emptyList()).withTimerInstances(emptyList()).withSvgContent(svgContent).build(),
                 ProcessInstanceDiagramSummary.builder().withProcessDefinition(ProcessSummary.builder().nodes(emptyList()).timers(emptyList()).build()).withNodeInstances(emptyList()).withTimerInstances(emptyList()).build()
         );
@@ -123,10 +140,10 @@ public class ProcessInstanceDiagramPresenterTest {
                                                                               "_3"));
 
         List<NodeInstanceSummary> nodeInstances = Arrays.asList(
-                NodeInstanceSummary.builder().withId(1l).withName("name-1").withType("HumanTask").withCompleted(false).build(),
-                NodeInstanceSummary.builder().withId(2l).withName(" ").withType("Split").withCompleted(false).build(),
-                NodeInstanceSummary.builder().withId(3l).withName("name-3").withType("HumanTask").withCompleted(true).build(),
-                NodeInstanceSummary.builder().withId(4l).withName(" ").withType("End").withCompleted(true).build()
+                NodeInstanceSummary.builder().withId(1l).withName("name-1").withNodeUniqueName("_1").withType("HumanTask").withCompleted(false).build(),
+                NodeInstanceSummary.builder().withId(2l).withName(" ").withNodeUniqueName("_2").withType("Split").withCompleted(false).build(),
+                NodeInstanceSummary.builder().withId(3l).withName("name-3").withNodeUniqueName("_3").withType("HumanTask").withCompleted(true).build(),
+                NodeInstanceSummary.builder().withId(4l).withName(" ").withNodeUniqueName("_4").withType("End").withCompleted(true).build()
         );
 
         List<TimerInstanceSummary> timerInstances = Arrays.asList(
@@ -139,7 +156,7 @@ public class ProcessInstanceDiagramPresenterTest {
         summary.setNodeInstances(nodeInstances);
         summary.setTimerInstances(timerInstances);
 
-        when(processService.getProcessInstanceDiagramSummary(processInstance.getProcessInstanceKey())).thenReturn(summary);
+        when(processService.getProcessInstanceDiagramSummary(eq(processInstance.getProcessInstanceKey()), any(), any(), any())).thenReturn(summary);
 
         presenter.setProcessInstance(processInstance);
 
@@ -217,6 +234,11 @@ public class ProcessInstanceDiagramPresenterTest {
         assertThat(viewTimerInstances.get(1).getName()).isEqualTo("t2");
         assertThat(viewTimerInstances.get(1).getDescription()).startsWith("NextExecution");
         assertThat(viewTimerInstances.get(1).getCallbacks()).hasSize(1);
+
+        ArgumentCaptor<Map> badgesInstancesCaptor = ArgumentCaptor.forClass(Map.class);
+        verify(view).setNodeBadges(badgesInstancesCaptor.capture());
+        final Map<String, Long> badges = badgesInstancesCaptor.getValue();
+        assertThat(badges).containsExactly(immutableEntry("_1", 1l), immutableEntry("_2", 1l), immutableEntry("_3", 1l), immutableEntry("_4", 1l));
     }
 
     @Test
@@ -264,7 +286,7 @@ public class ProcessInstanceDiagramPresenterTest {
         summary.setNodeInstances(nodeInstances);
         summary.setTimerInstances(timerInstances);
 
-        when(processService.getProcessInstanceDiagramSummary(processInstance.getProcessInstanceKey())).thenReturn(summary);
+        when(processService.getProcessInstanceDiagramSummary(eq(processInstance.getProcessInstanceKey()), any(), any(), any())).thenReturn(summary);
 
         presenter.setProcessInstance(processInstance);
 
@@ -273,6 +295,68 @@ public class ProcessInstanceDiagramPresenterTest {
         verify(view).setValue(timerNode);
         verify(view).setNodeInstances(singletonList(timerNodeInstance));
         verify(view).setTimerInstances(singletonList(timerInstance));
+        verify(view).setNodeBadges(any());
+        verify(view).onShow();
+    }
+
+    @Test
+    public void testOnDiagramNodeSelectedByDoubleClick() {
+        ProcessInstanceSummary processInstance = ProcessInstanceSummary.builder().withServerTemplateId("serverTemplateId").withDeploymentId("containerId").withProcessInstanceId(1L).withState(ProcessInstance.STATE_ACTIVE).build();
+
+        ProcessNodeSummary timerNode = new ProcessNodeSummary(1L, "name-1", "Timer", "_11");
+        List<ProcessNodeSummary> nodes = Arrays.asList(new ProcessNodeSummary(0L, " ", "Start", "_0"), timerNode,
+                                                       new ProcessNodeSummary(2L, " ", "Split", "_2"));
+
+        NodeInstanceSummary timerNodeInstance = NodeInstanceSummary.builder().withId(1l).withName("name-1").withNodeUniqueName("_2").withType("Timer").withCompleted(false).build();
+
+        List<NodeInstanceSummary> nodeInstances = Arrays.asList(
+                timerNodeInstance,
+                NodeInstanceSummary.builder().withId(2L).withName(" ").withNodeUniqueName("_1").withType("SubProcessNode").withCompleted(false).build(),
+                NodeInstanceSummary.builder().withId(3L).withName("name-3").withNodeUniqueName("_3").withType("HumanTask").withCompleted(true).build(),
+                NodeInstanceSummary.builder().withId(4L).withName(" ").withNodeUniqueName("_4").withType("End").withCompleted(true).build()
+        );
+
+        TimerInstanceSummary timerInstance = TimerInstanceSummary.builder().withId(1l).withTimerId(0l).withName("t1").build();
+
+        List<TimerInstanceSummary> timerInstances = Arrays.asList(
+                timerInstance,
+                TimerInstanceSummary.builder().withId(2L).withTimerId(1l).withName("t2").build()
+        );
+
+        ProcessInstanceDiagramSummary summary = new ProcessInstanceDiagramSummary();
+        summary.setProcessDefinition(ProcessSummary.builder().nodes(nodes).timers(singletonList(TimerSummary.builder().id(0l).uniqueId("_1").build())).build());
+        summary.setNodeInstances(nodeInstances);
+        summary.setTimerInstances(timerInstances);
+
+        when(processService.getProcessInstanceDiagramSummary(eq(processInstance.getProcessInstanceKey()), any(), any(), any())).thenReturn(summary);
+
+        presenter.setProcessInstance(processInstance);
+
+        presenter.onDiagramNodeSelectedByDoubleClick("_1");
+
+        verify(view, never()).setNodeInstances(singletonList(timerNodeInstance));
+        ArgumentCaptor<Command> captureCommand = ArgumentCaptor.forClass(Command.class);
+        verify(confirmPopup).show(any(), any(), any(), captureCommand.capture());
+        captureCommand.getValue().execute();
+        verify(processInstanceEvent).fire(any());
+        verify(view, never()).setTimerInstances(singletonList(timerInstance));
+        verify(view).onShow();
+
+        List<NodeInstanceSummary> nodeInstancesMultipleSubProcess = Arrays.asList(
+                timerNodeInstance,
+                NodeInstanceSummary.builder().withId(2L).withName("one").withNodeUniqueName("_1").withType("SubProcessNode").withCompleted(false).build(),
+                NodeInstanceSummary.builder().withId(5L).withName("two").withNodeUniqueName("_1").withType("SubProcessNode").withCompleted(false).build(),
+                NodeInstanceSummary.builder().withId(3L).withName("name-3").withNodeUniqueName("_3").withType("HumanTask").withCompleted(true).build(),
+                NodeInstanceSummary.builder().withId(4L).withName(" ").withNodeUniqueName("_4").withType("End").withCompleted(true).build()
+        );
+        summary.setNodeInstances(nodeInstancesMultipleSubProcess);
+        when(processService.getProcessInstanceDiagramSummary(eq(processInstance.getProcessInstanceKey()), anyString(), anyString(), anyString())).thenReturn(summary);
+        presenter.setProcessInstance(processInstance);
+
+        presenter.onDiagramNodeSelectedByDoubleClick("_1");
+        verify(view, never()).setNodeInstances(singletonList(timerNodeInstance));
+        verify(alertPopup).alert(any(), any());
+        verify(view, never()).setTimerInstances(singletonList(timerInstance));
     }
 
     @Test
@@ -298,7 +382,7 @@ public class ProcessInstanceDiagramPresenterTest {
         summary.setNodeInstances(emptyList());
         summary.setTimerInstances(emptyList());
 
-        when(processService.getProcessInstanceDiagramSummary(processInstance.getProcessInstanceKey())).thenReturn(summary);
+        when(processService.getProcessInstanceDiagramSummary(eq(processInstance.getProcessInstanceKey()), any(), any(), any())).thenReturn(summary);
 
         presenter.setProcessInstance(processInstance);
 
@@ -314,21 +398,12 @@ public class ProcessInstanceDiagramPresenterTest {
     public void testOnNodeInstanceCancelled() {
         ProcessInstanceSummary processInstance = ProcessInstanceSummary.builder().withServerTemplateId("serverTemplateId").withDeploymentId("containerId").withProcessInstanceId(1l).withState(ProcessInstance.STATE_ACTIVE).build();
 
-        NodeInstanceSummary humanTask = NodeInstanceSummary.builder().withId(1l).withName("name-1").withType("HumanTask").withCompleted(false).build();
+        NodeInstanceSummary humanTask = NodeInstanceSummary.builder().withId(1l).withName("name-1").withNodeUniqueName("_1").withType("HumanTask").withCompleted(false).build();
 
-        List<NodeInstanceSummary> nodeInstances = Arrays.asList(
-                humanTask,
-                NodeInstanceSummary.builder().withId(2l).withName(" ").withType("Split").withCompleted(false).build(),
-                NodeInstanceSummary.builder().withId(3l).withName("name-3").withType("HumanTask").withCompleted(true).build(),
-                NodeInstanceSummary.builder().withId(4l).withName(" ").withType("End").withCompleted(true).build()
-        );
+        ProcessInstanceDiagramSummary summary = createHumanTaskProcessInstanceDiagramSummary(humanTask);
+        new ProcessInstanceDiagramSummary();
 
-        ProcessInstanceDiagramSummary summary = new ProcessInstanceDiagramSummary();
-        summary.setProcessDefinition(ProcessSummary.builder().nodes(emptyList()).timers(emptyList()).build());
-        summary.setNodeInstances(nodeInstances);
-        summary.setTimerInstances(emptyList());
-
-        when(processService.getProcessInstanceDiagramSummary(processInstance.getProcessInstanceKey())).thenReturn(summary);
+        when(processService.getProcessInstanceDiagramSummary(eq(processInstance.getProcessInstanceKey()), any(), any(), any())).thenReturn(summary);
 
         presenter.setProcessInstance(processInstance);
 
@@ -343,21 +418,10 @@ public class ProcessInstanceDiagramPresenterTest {
     public void testOnNodeInstanceReTriggered() {
         ProcessInstanceSummary processInstance = ProcessInstanceSummary.builder().withServerTemplateId("serverTemplateId").withDeploymentId("containerId").withProcessInstanceId(1l).withState(ProcessInstance.STATE_ACTIVE).build();
 
-        NodeInstanceSummary humanTask = NodeInstanceSummary.builder().withId(1l).withName("name-1").withType("HumanTask").withCompleted(false).build();
+        NodeInstanceSummary humanTask = NodeInstanceSummary.builder().withId(1l).withName("name-1").withNodeUniqueName("_1").withType("HumanTask").withCompleted(false).build();
+        ProcessInstanceDiagramSummary summary = createHumanTaskProcessInstanceDiagramSummary(humanTask);
 
-        List<NodeInstanceSummary> nodeInstances = Arrays.asList(
-                humanTask,
-                NodeInstanceSummary.builder().withId(2l).withName(" ").withType("Split").withCompleted(false).build(),
-                NodeInstanceSummary.builder().withId(3l).withName("name-3").withType("HumanTask").withCompleted(true).build(),
-                NodeInstanceSummary.builder().withId(4l).withName(" ").withType("End").withCompleted(true).build()
-        );
-
-        ProcessInstanceDiagramSummary summary = new ProcessInstanceDiagramSummary();
-        summary.setProcessDefinition(ProcessSummary.builder().nodes(emptyList()).timers(emptyList()).build());
-        summary.setNodeInstances(nodeInstances);
-        summary.setTimerInstances(emptyList());
-
-        when(processService.getProcessInstanceDiagramSummary(processInstance.getProcessInstanceKey())).thenReturn(summary);
+        when(processService.getProcessInstanceDiagramSummary(eq(processInstance.getProcessInstanceKey()), any(), any(), any())).thenReturn(summary);
 
         presenter.setProcessInstance(processInstance);
 
@@ -383,7 +447,7 @@ public class ProcessInstanceDiagramPresenterTest {
         summary.setNodeInstances(emptyList());
         summary.setTimerInstances(timerInstance);
 
-        when(processService.getProcessInstanceDiagramSummary(processInstance.getProcessInstanceKey())).thenReturn(summary);
+        when(processService.getProcessInstanceDiagramSummary(eq(processInstance.getProcessInstanceKey()), any(), any(), any())).thenReturn(summary);
 
         presenter.setProcessInstance(processInstance);
 
@@ -411,7 +475,7 @@ public class ProcessInstanceDiagramPresenterTest {
         summary.setNodeInstances(emptyList());
         summary.setTimerInstances(Arrays.asList(t1, t2));
 
-        when(processService.getProcessInstanceDiagramSummary(processInstance.getProcessInstanceKey())).thenReturn(summary);
+        when(processService.getProcessInstanceDiagramSummary(eq(processInstance.getProcessInstanceKey()), any(), any(), any())).thenReturn(summary);
 
         presenter.setProcessInstance(processInstance);
 
@@ -446,4 +510,98 @@ public class ProcessInstanceDiagramPresenterTest {
                                                                                     "TimerNode",
                                                                                     "_1")));
     }
+
+    @Test
+    public void testCustomColorFromPreferences() {
+        String completeNodeColor = "#888888";
+        String completeNodeBorderColor = "#888887";
+        String activeNodeBorderColor = "#888886";
+        ProcessInstanceSummary processInstance = ProcessInstanceSummary.builder().withServerTemplateId("serverTemplateId").withDeploymentId("containerId").withProcessInstanceId(1l).withState(ProcessInstance.STATE_ACTIVE).build();
+        ProcessInstanceDiagramSummary summary = createHumanTaskProcessInstanceDiagramSummary(null);
+
+        when(preferences.getProcessInstanceDiagramCompletedNodeColor()).thenReturn(completeNodeColor);
+        when(preferences.getProcessInstanceDiagramCompletedNodeBorderColor()).thenReturn(completeNodeBorderColor);
+        when(preferences.getProcessInstanceDiagramActiveNodeBorderColor()).thenReturn(activeNodeBorderColor);
+        when(processService.getProcessInstanceDiagramSummary(eq(processInstance.getProcessInstanceKey()), anyString(), anyString(), anyString())).thenReturn(summary);
+
+        presenter.setProcessInstance(processInstance);
+
+        verify(processService).getProcessInstanceDiagramSummary(processInstance.getProcessInstanceKey(), completeNodeColor, completeNodeBorderColor, activeNodeBorderColor);
+    }
+
+    private ProcessInstanceDiagramSummary createHumanTaskProcessInstanceDiagramSummary(NodeInstanceSummary humanTask) {
+        if (humanTask == null) {
+            humanTask = NodeInstanceSummary.builder().withId(1l).withName("name-1").withNodeUniqueName("_1").withType("HumanTask").withCompleted(false).build();
+        }
+
+        List<NodeInstanceSummary> nodeInstances = Arrays.asList(
+                humanTask,
+                NodeInstanceSummary.builder().withId(2l).withName(" ").withNodeUniqueName("_2").withType("Split").withCompleted(false).build(),
+                NodeInstanceSummary.builder().withId(3l).withName("name-3").withNodeUniqueName("_3").withType("HumanTask").withCompleted(true).build(),
+                NodeInstanceSummary.builder().withId(4l).withName(" ").withNodeUniqueName("_4").withType("End").withCompleted(true).build()
+        );
+
+        ProcessInstanceDiagramSummary summary = new ProcessInstanceDiagramSummary();
+        summary.setProcessDefinition(ProcessSummary.builder().nodes(emptyList()).timers(emptyList()).build());
+        summary.setNodeInstances(nodeInstances);
+        summary.setTimerInstances(emptyList());
+        return summary;
+    }
+
+    @Test
+    public void testLoadProcessInstanceDetails() {
+        ProcessInstanceDiagramSummary summary = createHumanTaskProcessInstanceDiagramSummary(null);
+
+        ProcessInstanceSummary processInstance = ProcessInstanceSummary.builder().withServerTemplateId("serverTemplateId").withDeploymentId("containerId").withProcessInstanceId(1L).withState(ProcessInstance.STATE_ACTIVE).build();
+
+        String completeNodeColor = "#888888";
+        String completeNodeBorderColor = "#888887";
+        String activeNodeBorderColor = "#888886";
+        when(preferences.getProcessInstanceDiagramCompletedNodeColor()).thenReturn(completeNodeColor);
+        when(preferences.getProcessInstanceDiagramCompletedNodeBorderColor()).thenReturn(completeNodeBorderColor);
+        when(preferences.getProcessInstanceDiagramActiveNodeBorderColor()).thenReturn(activeNodeBorderColor);
+        when(processService.getProcessInstanceDiagramSummary(eq(processInstance.getProcessInstanceKey()), anyString(), anyString(), anyString())).thenReturn(summary);
+
+        presenter.setProcessInstance(processInstance);
+
+        verify(view, times(2)).setProcessNodes(any());
+        verify(view).setShowOrHideNodeActionsCommand(any());
+        verify(view).setShowOrHideParentAndSubProcessPanelCommand(any());
+        verify(view).showOrHideParentAndSubProcessPanelTriggered();
+        verify(view, never()).expandDiagram();
+        verify(view, never()).disableExpandAnchor();
+        verify(view).showOrHideNodeActionsTriggered();
+    }
+
+    @Test
+    public void testShowOrHideNodeActions() {
+        ProcessInstanceSummary processInstanceActive = ProcessInstanceSummary.builder().withServerTemplateId("serverTemplateId").withDeploymentId("containerId").withState(ProcessInstance.STATE_ACTIVE).build();
+        presenter.showOrHideNodeActions(false, processInstanceActive);
+
+        verify(view).showNodeActions();
+
+        ProcessInstanceSummary processInstanceCompleted = ProcessInstanceSummary.builder().withServerTemplateId("serverTemplateId").withDeploymentId("containerId").withState(ProcessInstance.STATE_COMPLETED).build();
+        presenter.showOrHideNodeActions(false, processInstanceCompleted);
+        verify(view).hideNodeActions();
+    }
+
+    @Test
+    public void testShowOrHideParentAndSubProcessPanel() {
+        ProcessInstanceDiagramSummary summary = createHumanTaskProcessInstanceDiagramSummary(null);
+
+        ProcessInstanceSummary processInstance = ProcessInstanceSummary.builder().withServerTemplateId("serverTemplateId").withDeploymentId("containerId").withProcessInstanceId(1l).withState(ProcessInstance.STATE_ACTIVE).build();
+
+        summary.setParentProcessInstanceSummary(ProcessInstanceSummary.builder().withServerTemplateId("serverTemplateId").withDeploymentId("containerId")
+                                                        .withProcessInstanceId(10l).withProcessName("parentProcess").build());
+        List<ProcessInstanceSummary> list = Arrays.asList(ProcessInstanceSummary.builder().withParentId(10l).withServerTemplateId("serverTemplateId").withDeploymentId("containerId")
+                                                                  .withProcessId("sub-process-one").withProcessInstanceId(11l).withProcessName("sub-process-one").build(),
+                                                          ProcessInstanceSummary.builder().withParentId(10l).withServerTemplateId("serverTemplateId").withDeploymentId("containerId")
+                                                                  .withProcessId("sub-process-two").withProcessInstanceId(12l).withProcessName("sub-process-two").build());
+
+        presenter.showOrHideParentAndSubProcessPanel(processInstance, list);
+        verify(view).showParentAndSubProcessPanel();
+
+        presenter.showOrHideParentAndSubProcessPanel(null, Collections.emptyList());
+        verify(view).hideParentAndSubProcessPanel();
+    }
 }
